<?php

declare(strict_types=1);

namespace Dedi\SyliusSettingsPlugin\DependencyInjection\Compiler;

use Dedi\Contracts\SyliusPluginSettings\SettingsProviderRegistryInterface;
use Dedi\SyliusSettingsPlugin\DependencyInjection\Configuration;
use Dedi\SyliusSettingsPlugin\Model\Config;
use Dedi\SyliusSettingsPlugin\Provider\Decorator;
use MonsieurBiz\SyliusSettingsPlugin\Provider\SettingsProviderInterface;
use Symfony\Component\Config\Definition\Processor;
use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Reference;

class SettingsProviderPass implements CompilerPassInterface
{
    public function process(ContainerBuilder $container)
    {
        $processor = new Processor();
        $configuration = $processor->processConfiguration(new Configuration(), $container->getExtensionConfig('dedi_sylius_settings'));

        foreach ($configuration['plugins'] as $pluginName => $configurationValues) {
            $configId = 'dedi.sylius_settings.plugins.' . $pluginName . '.config';
            $container
                ->register($configId)
                ->setClass(Config::class)
                ->setArguments([
                    $pluginName,
                    $configurationValues['model'],
                    $configurationValues['factory'],
                    $configurationValues['provider'],
                    $configurationValues['permission'],
                    $configurationValues['keys'],
                ]);

            $decoratorId = 'dedi.sylius_settings.plugins.' . $pluginName . '.provider';
            $container
                ->register($decoratorId, Decorator::class)
                ->setDecoratedService($configurationValues['provider'])
                ->setArgument('$decoratedSettingsProvider', new Reference($decoratorId . '.inner'))
                ->setArgument('$settingsFactory', new Reference($configurationValues['factory']))
                ->setArgument('$monsieurBizSettingsProvider', new Reference(SettingsProviderInterface::class))
                ->setArgument('$propertyAccessor', new Reference('property_accessor'))
                ->setArgument('$config', new Reference($configId))
                ->addTag('dedi.settings_provider')
            ;
        }

        if (!$container->has(SettingsProviderRegistryInterface::class)) {
            return;
        }

        $definition = $container->findDefinition(SettingsProviderRegistryInterface::class);
        $taggedServices = $container->findTaggedServiceIds('dedi.settings_provider');

        foreach ($taggedServices as $id => $tags) {
            $definition->addMethodCall('register', [new Reference($id)]);
        }
    }
}
