<?php

declare(strict_types=1);

namespace Dedi\SyliusSettingsPlugin\Security\Voter;

use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Authorization\Voter\Voter;

class CanConfigurePluginsVoter extends Voter
{
    public function __construct(private readonly array $allowedRoles = [])
    {
    }

    protected function supports(string $attribute, mixed $subject): bool
    {
        return $attribute === 'CAN_CONFIGURE_PLUGINS';
    }

    protected function voteOnAttribute(string $attribute, mixed $subject, TokenInterface $token): bool
    {
        $user = $token->getUser();
        if (null === $user) {
            return false;
        }

        foreach ($user->getRoles() as $role) {
            if (in_array($role, $this->allowedRoles, true)) {
                return true;
            }
        }

        return false;
    }
}
